#include "../include/display.h"

/**
 * Draws the display to the SDL renderer
 * If the bool in the display is true, a white pixel will be written on that position
 * The redraw flag is set to false afterwards
 *
 * @param renderer The SDL renderer
 */
void Display::draw(SDL_Renderer * renderer) {
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
    SDL_RenderClear(renderer);
    SDL_SetRenderDrawColor(renderer, 255, 255, 255, 255);

    for (int y = 0; y < 32; y++) {
        for (int x = 0; x < 64; x++) {
            if (display[(y * 64) + x] == STATE_ON) {
                SDL_Rect r;
                r.x = x + 10;
                r.y = y + 10;
                r.w = 10;
                r.h = 10;
                SDL_RenderFillRect(renderer, &r);
            }
        }
    }

    SDL_RenderPresent(renderer);
    this->flagRedraw = false;
}

bool Display::needsRedraw() const {
    return this->flagRedraw;
}

/**
 * Clears the display by setting all pixels to false
 */
void Display::clear() {
    std::fill(display, display + 64 * 32, STATE_OFF);
    this->flagRedraw = true;
}
